/// <reference types="mapbox-gl" />
/// <reference types="geojson" />
/// <reference types="./typings/axios" />

export = MapxusMap;

declare namespace MapxusMap {
  /**
   * Map
   */
  export class Map extends Evented {
    constructor(option: IMapOption);

    renderComplete(callback: Function): void;

    getIndoor(): Indoor;

    getMap(): mapboxgl.Map;

    transformPoiTextFieldLanguage(lanaguage: string): void;

    enableSwitch(): Map;

    disableSwitch(): Map;

    enableSwitchBuilding(): Map;

    disableSwitchBuilding(): Map;

    hiddenOutdoorLayer(isHiddenOutdoor: boolean): void;

    floors(): string[];

    currentFloor(): string;

    building(): IBuilding;

    getFeature(id: string): GeoJSON.Feature<GeoJSON.GeometryObject, IBuilding>;

    selectBuilding(
        feature: GeoJSON.Feature<GeoJSON.GeometryObject, IBuilding>
    ): Map;

    switchBuilding(buildingId: string, callback?: Function): Map;

    switchFloor(floor: string, callback?: Function): Map;

    onMapClickListener(listener: Function): Map;

    onIndoorPoiListener(listener: Function): Map;

    onFloorChangeListener(listener: Function): Map;

    onBuildingChangeListener(listener: Function): Map;

    disableFloorControl(): Map;

    enableFloorControl(
        position?: 'top-right' | 'top-left' | 'bottom-right' | 'bottom-left'
    ): Map;

    disableBuildingSelector(): Map;

    enableBuildingSelector(
        position?: 'top-right' | 'top-left' | 'bottom-right' | 'bottom-left'
    ): Map;

    switchTheme(theme: string): void;
  }

  /**
   * Indoor
   */
  export class Indoor extends Evented {
    constructor(map: mapboxgl.Map, isHiddenOutdoor?: boolean);

    enableSwitch(): Indoor;

    disableSwitch(): Indoor;

    enableSwitchBuilding(): Indoor;

    disableSwitchBuilding(): Indoor;

    floors(): string[];

    groundFloor(): string;

    buildingId(): string;

    currentFloor(): string;

    features(): GeoJSON.Feature<GeoJSON.GeometryObject, IBuilding>[];

    getFeature(id?: string): GeoJSON.Feature<GeoJSON.GeometryObject, IBuilding>;

    getBuildingByPoint(
        point: mapboxgl.PointLike
    ): GeoJSON.Feature<GeoJSON.GeometryObject, IBuilding>;

    selectBuilding(
        feature: GeoJSON.Feature<GeoJSON.GeometryObject, IBuilding>
    ): void;

    switchBuilding(
        feature: GeoJSON.Feature<
            GeoJSON.GeometryObject | GeoJSON.Polygon,
            IBuilding
            >
    ): void;

    goToBuilding(buildingId: string, callback?: Function): void;

    switchFloor(floor: string): void;

    initLayer(): void;

    hiddenOutdoorLayer(isHiddenOutdoor: boolean): void;
  }

  export class Marker extends Evented {
    constructor(map?: Map);

    onEventListener(type: string, callback: Function): void;

    addTo(map: Map): Marker;

    create(
        coordinates: number[] | number[][],
        floor?: string,
        buildingId?: string
    ): Marker;

    remove(): Marker;

    switch(floor: string): Marker;

    layerId(): string;

    setImageUrl(imageUrl: string): Marker;

    setImageScale(imageScale: number): Marker;
  }

  export class Poi {
    constructor(map: mapboxgl.Map);

    pointClickFunction(listener: Function): void;
  }

  export class FloorsControl implements mapboxgl.IControl {
    setMap(map: Map): void;

    onAdd(map: mapboxgl.Map): HTMLElement;

    onRemove(): void;

    getDefaultPosition(): string;

    floorClickFunction(listener: Function): void;

    switchFloor(floor: string, callback?: Function): void;
  }

  export class BuildingFilterControl implements mapboxgl.IControl {
    setMap(map: Map): void;

    setPosition(position: string | string[]): void;

    onAdd(map: mapboxgl.Map): HTMLElement;

    onRemove(): void;

    getDefaultPosition(): string;
  }

  export class BuildingsService {
    constructor();

    searchByIds(
        ...id: string[]
    ): AxiosType.AxiosPromise<IResponse<IResponseBuildings>>;

    searchByDistance(
        keywords: string,
        center: number[],
        distance: number,
        offset: number,
        page: number
    ): AxiosType.AxiosPromise<IResponse<IResponseBuildings>>;

    searchByBbox(
        keywords: string,
        bbox: number[],
        offset: number,
        page: number
    ): AxiosType.AxiosPromise<IResponse<IResponseBuildings>>;

    searchByGlobal(
        keywords: string,
        offset: number,
        page: number
    ): AxiosType.AxiosPromise<IResponse<IResponseBuildings>>;
  }

  export class PoisService {
    constructor();

    searchByIds(id: string[]): AxiosType.AxiosPromise<IResponse<IResponsePois>>;

    searchByDistance(
        keywords: string,
        center: number[],
        distance: number,
        offset: number,
        page: number
    ): AxiosType.AxiosPromise<IResponse<IResponsePois>>;

    searchByBbox(
        keywords: string,
        bbox: number[],
        offset: number,
        page: number
    ): AxiosType.AxiosPromise<IResponse<IResponsePois>>;

    searchByBuildingId(
        keywords: string,
        buildingId: string,
        offset: number,
        page: number
    ): AxiosType.AxiosPromise<IResponse<IResponsePois>>;

    searchCategory(
        buildingId: string,
        floor?: string
    ): AxiosType.AxiosPromise<IResponse<string[]>>;

    searchOrientation(
        angle: number,
        buildingId: string,
        center: number[],
        distance: number,
        distanceSearchType: string,
        floor: string
    ): AxiosType.AxiosPromise<IResponse<IResponseOrientationPoi>>;
  }

  export class RouteService {
    constructor();

    search(
        fromCoord: number[],
        toCoord: number[],
        fromFloor: string,
        toFloor: string,
        fromBuildingId: string,
        toBuildingId: string,
        vehicle: string,
        locale?: string,
        toDoor?: boolean
    ): AxiosType.AxiosPromise<IResponse<IResponseRoute>>;
  }

  /**
   * Evented
   */
  export class Evented {
    on(type: string, listener: Function): this;

    off(type?: string | any, listener?: Function): this;

    once(type: string, listener: Function): this;

    fire(type: string, properties?: { [key: string]: any }): this;
  }

  export interface IMapOption {
    // mapbox instance
    map: mapboxgl.Map;

    appId: string;

    secret: string;

    enableFloorControl?: boolean;

    enableBuildingSelector?: boolean;
    // if specify buildingId, mapbox center and zoom would invalid
    buildingId?: string;
    // must dependent on buildingId
    floor?: string;

    poiId?: string;

    hiddenOutdoor?: boolean;

    theme?: ThemeType;
  }

  export interface IBuilding {
    building: string;

    fids?: string;

    floors: string;

    ground_floor: string;

    id: string;

    name: string;

    type: string;

    labelCenter?: ILocation;
  }

  export interface IPoi {
    address?: string;

    buildingId: string;

    category?: string;

    description?: string;

    email?: string;

    floor: string;

    id: string;

    location: ILocation;

    name: object;

    openingHours?: string;

    osmRefId?: number;

    phone?: string;

    poiId?: string;

    website?: string;
  }

  export interface ILocation {
    lat: number;

    lon: number;
  }

  export interface IResponse<T> {
    code: number;

    message: string;

    result: T;
  }

  export interface IResponseBuildings {
    buildings: IBuilding[];

    total: number;
  }

  export interface IResponsePois {
    pois: IPoi[];

    total: number;
  }

  export interface IResponseRoute {
    hints: IRouteHints;
    info: IRouteInfo;
    paths: IRoutePaths[];
  }

  interface IRouteHints {
    'visited_nodes.average': string;
    'visited_nodes.sum': string;
  }

  interface IRouteInfo {
    copyrights: string[];
    took: number;
  }

  interface IOrientationPoi {
    angle: number;
    buildingId: string;
    distance: number;
    floor: string;
    location: ILocation;
    name: object;
    poiId: string;
  }

  interface IResponseOrientationPoi {
    orientationPois: IOrientationPoi[];
    total: number;
  }

  export interface IRoutePaths {
    ascend: number;
    bbox: number[];
    descend: number;
    details: object;
    distance: number;
    instructions: any;
    legs: number[];
    points: string | GeoJSON.FeatureCollection<GeoJSON.LineString>;
    points_encoded: boolean;
    snapped_waypoints: string;
    time: number;
    transfers: number;
    weight: number;
  }

  export enum DistanceSearchType {
    POINT = 'Point',
    POLYGON = 'Polygon'
  }

  export enum ThemeType {
    CHRISTMAS = 'christmas',
    HALLOWEEN = 'halloween',
    MAPPY_BEE = 'mappyBee',
    MAPXUS = 'mapxus',
    LANDS_D = 'landsD',
    COMMON = 'common'
  }

  export enum WayFidingVehicleType {
    FOOT = 'foot',
    WHEELCHAIR = 'wheelchair'
  }
}